/*
 * linux/include/asm-arm/arch-clps711x/ide.h
 */
#include <linux/config.h>
#include <asm/irq.h>
#include <asm/io.h>


#define PHAT_IDE_DEBUG 1


/*
 * Set up a hw structure for a specified data port, control port and IRQ.
 * This should follow whatever the default interface uses.
 */
static __inline__ void
ide_init_hwif_ports(hw_regs_t *hw, int data_port, int ctrl_port, int *irq)
{
	ide_ioreg_t reg = (ide_ioreg_t) data_port;
	int i;

	memset(hw, 0, sizeof(*hw));

	for (i = IDE_DATA_OFFSET; i <= IDE_STATUS_OFFSET; i++) {
		hw->io_ports[i] = reg;
		reg += 1;
	}
	hw->io_ports[IDE_CONTROL_OFFSET] = (ide_ioreg_t) ctrl_port;
	if (irq != NULL)
		*irq = 0;
}

/*
 * This registers the standard ports for this architecture with the IDE
 * driver.
 */

#ifdef CONFIG_ARCH_PHATNOISE
#include <asm/hardware.h>
#include <asm/arch/hardware.h>
#endif

static __inline__ void
ide_init_default_hwifs(void)
{
#ifdef CONFIG_ARCH_PHATNOISE
	hw_regs_t hw;
	
	printk("PhatNoise ide_init_default_hwifs()\n");
	/*
	 * Make sure external expansion clock is enabled.
	 */
	IO_SYSCON1 |= SYSCON1_EXCKEN;
#if PHAT_IDE_DEBUG
	printk("IO_SYSCON1 = 0x%08X\n", (unsigned) IO_SYSCON1);
	printk("IO_SYSCON2 = 0x%08X\n", (unsigned) IO_SYSCON2);
	printk("IO_SYSCON3 = 0x%08X\n", (unsigned) IO_SYSCON3);

	printk("IO_MEMCFG1 = 0x%08X\n", (unsigned) IO_MEMCFG1);
	printk("IO_MEMCFG2 = 0x%08X\n", (unsigned) IO_MEMCFG2);
#endif

	ide_init_hwif_ports(&hw, PHAT_VIRT_IDE, PHAT_VIRT_IDE+0x0E, NULL);
	//ide_init_hwif_ports(&hw, PHAT_PHYS_IDE, PHAT_PHYS_IDE+0x0E, NULL);
	hw.irq		= IRQ_EINT2;
	ide_register_hw(&hw, NULL);
#endif          
}



/* define our IN_BYTE and OUT_BYTE for ide.  
 *
 */
#if 1 && CONFIG_ARCH_PHATNOISE
#ifndef DEFINE_ARCH_IDE_IN_OUT
#define DEFINE_ARCH_IDE_IN_OUT 1


static uint8_t inb_ide(unsigned int p)
{
	unsigned int r;

#ifdef RIDICULOUS_IDE_DEBUG
	static unsigned int lastp = 0;
	if (p != lastp) {
		printk(KERN_WARNING "inb_ide(0x%x)\n", (unsigned) p);
	}
#endif

	r = (* (volatile unsigned int *)(p));
	//r = inb(p) ;

#ifdef RIDICULOUS_IDE_DEBUG
	if (p != lastp) {
		printk(KERN_WARNING "inb_ide(0x%x) = 0x%x\n", 
				(unsigned) p, r);
		lastp = p;
	}
#endif
	if ((p & 0x3) == 1)
	{
		r = r >> 24;
	}
	if ((p & 0x3) == 2)
	{
		r = r >> 16;
	}
	if ((p & 0x3) == 3)
	{
		r = r >> 8;
	}
	return (uint8_t) r; 
}

#define HAVE_ARCH_IN_BYTE 1
#define IN_BYTE(p)	(byte)inb_ide(p)



static void outb_ide(byte b, unsigned int p)
{
#ifdef RIDICULOUS_IDE_DEBUG
	static unsigned int lastp = 0;
	static unsigned int lastb = 0;

	if (p != lastp || (p == lastp && (b != lastb))) {
		printk(KERN_WARNING "outb_ide(%u, 0x%x)\n", 
				b, (unsigned) p);
		lastp = p;
		lastb = b;
	}
#endif

	//outb_p(b, p);
	*(volatile byte *) p = b;
}

#define HAVE_ARCH_OUT_BYTE 1
#define OUT_BYTE(b,p)	outb_ide((b),(p))

static uint16_t inw_ide(int p)
{
	return (uint16_t) (* (volatile uint32_t *) p);
}

static void outw_ide(uint16_t b, int p)
{
	* (volatile uint32_t *) p = b;
}



#endif
#endif


